<?php
/**
 *  Copyright 2010 KLARNA AB. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without modification, are
 *  permitted provided that the following conditions are met:
 *
 *     1. Redistributions of source code must retain the above copyright notice, this list of
 *        conditions and the following disclaimer.
 *
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list
 *        of conditions and the following disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 *  THIS SOFTWARE IS PROVIDED BY KLARNA AB "AS IS" AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KLARNA AB OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 *  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  The views and conclusions contained in the software and documentation are those of the
 *  authors and should not be interpreted as representing official policies, either expressed
 *  or implied, of KLARNA AB.
 */
require_once(dirname(__FILE__).'/api/Klarna.php');

/**
 * Extension of the Klarna API to specify module version
 * and to add some utility methods.
 *
 * @since     ##KLARNA_VEYTON_DATE##
 * @version   1.0.3
 * @link      http://integration.klarna.com/
 * @copyright Copyright (c) 2005-2011 Klarna AB (http://klarna.com)
 */
class XtKlarna extends Klarna
{

    /**
     * Class constructor
     */
    public function __construct()
    {
        $this->VERSION = 'php:veyton:1.0.3';
    }

    /**
    * Attempt to guess country from currency
    */
    private function getCountryForCurrency($currency) {
        switch (strtolower($currency)) {
            case "sek":
                return "se";
            case "nok":
                return "no";
            case "dkk":
                return "dk";
            default:
                return _STORE_COUNTRY;
        }
    }

    /**
     * Checks if the payment option(s) should be shown.
     *
     * @return bool
     */
    public function showPayment()
    {
        global $currency, $language;

        $customer = $_SESSION['customer'];

        //Country, logged in? otherwise store default.
        $country = _STORE_COUNTRY;
        if ($customer->customers_id > 0) {
            if (isset($customer->customer_shipping_address['customers_country_code'])) {
                $country = $customer->customer_shipping_address['customers_country_code'];
            } else if (isset($customer->customer_default_address['customers_country_code'])) {
                $country = $customer->customer_default_address['customers_country_code'];
            }
        } else {
            $country = self::getCountryForCurrency($currency->code);
        }
        try {
            $this->setCountry($country);
            $this->setCurrency($currency->code);

            //Disregard language?
            //$this->setLanguage($language->code);
            Klarna::printDebug('showPayment', $country.','.$currency->code);
            return $this->checkCountryCurrency();
        }
        catch(Exception $e) {
            Klarna::printDebug('showPayment error', $e);
            return false;
        }
    }

    /**
     * Check Gender
     *
     * @param string $gender	Check male(1) or female(0)
     *
     * @return bool
     */
    public static function validate_gender($gender)
    {
        if ($gender == '0' || $gender == '1') {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Check String
     *
     * @param string $string	Check length of a string and is it a string
     *
     * @return bool
     */
    public static function validate_string($string)
    {
        $string = trim($string);
        return (strlen($string) > 0);
    }

    /**
     * Check house number
     *
     * @param string $string	House number has minimum one character
     *
     * @return bool
     */
    public static function validate_house_number($house_number)
    {
        return self::validate_string($house_number);
    }

    /**
     * Check ZIP code
     *
     * @param string $zipcode	ZIP code has minimum one character
     *
     * @return bool
     */
    public static function validate_zipcode($zipcode)
    {
        return self::validate_string($zipcode);
    }

    /**
     * Check Salary
     *
     * @param string  $salary Salary is significant digit
     *
     * @return bool
     */
    public static function validate_salary ($salary)
    {
        //Salary is significant digit
        if (ctype_digit($salary) && $salary != '0') {
            return true;
        } else {
            return false;
        }
    }
}
