<?php
/**
 *  Copyright 2010 KLARNA AB. All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without modification, are
 *  permitted provided that the following conditions are met:
 *
 *     1. Redistributions of source code must retain the above copyright notice, this list of
 *        conditions and the following disclaimer.
 *
 *     2. Redistributions in binary form must reproduce the above copyright notice, this list
 *        of conditions and the following disclaimer in the documentation and/or other materials
 *        provided with the distribution.
 *
 *  THIS SOFTWARE IS PROVIDED BY KLARNA AB "AS IS" AND ANY EXPRESS OR IMPLIED
 *  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL KLARNA AB OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 *  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  The views and conclusions contained in the software and documentation are those of the
 *  authors and should not be interpreted as representing official policies, either expressed
 *  or implied, of KLARNA AB.
 */

defined('_VALID_CALL') or die('Direct Access is not allowed.');

require_once(dirname(__FILE__).'/api/Klarna.php');
require_once(dirname(__FILE__).'/class.klarna.php');
require_once(dirname(__FILE__).'/class.xt_format.php');
require_once(dirname(__FILE__).'/checkout/classes/class.KlarnaAPI.php');
require_once(dirname(__FILE__).'/checkout/classes/class.KlarnaProductPrice.php');

require_once(dirname(__FILE__).'/api/transport/xmlrpc-3.0.0.beta/lib/xmlrpc.inc');
require_once(dirname(__FILE__).'/api/transport/xmlrpc-3.0.0.beta/lib/xmlrpc_wrappers.inc');

/**
 * The Klarna module providing an integration over XMLRPC to Klarna's payment services.
 *
 * @since     ##KLARNA_VEYTON_DATE##
 * @version   1.0.3
 * @link      http://integration.klarna.com/
 * @copyright Copyright (c) 2005-2011 Klarna AB (http://klarna.com)
 */
class xt_klarna
{
    /**
     * Module version.
     *
     * @var string
     */
    public $version = '1.0.3';

    /**
     * If the module uses a gateway/redirect solution.
     *
     * @var bool
     */
    public $external = false;

    /**
     * If the module uses a iframe solution.
     *
     * @var bool
     */
    public $iframe = false;

    /**
     * If the module has sub payment modules.
     *
     * @var bool
     */
    public $subpayments = true;

    /**
     * An array of the allowed sub payment modules.
     *
     * @var array
     */
    public $allowed_subpayments;

    /**
     * Data available to the template, etc.
     *
     * @var array
     */
    public $data = array();

    /**
     * The Klarna API object.
     *
     * @var XtKlarna
     */
    public $api;

    /**
     * Merchant ID.
     *
     * @var int
     */
    protected $_eid = 0;

    /**
     * Shared secret.
     *
     * @var type
     */
    protected $_secret = '';

    /**
     * Which mode (BETA/LIVE) is being used.
     *
     * @var int
     */
    protected $_mode = 0;

    /**
     * Input fields for the standard checkout.
     *
     * @var array
     */
    protected $_inputs = array();

    /**
     * Location of the standardized checkout.
     *
     * @var string
     */
    protected $_checkoutDir;

    /**
     * URL to the plugin location.
     *
     * @var string
     */
    protected $_webRoot;

    /**
     * An array of error messages related to input field.
     *
     * @var array
     */
    protected $_errorMsg;

    /**
     * An array of current errors.
     *
     * @var array
     */
    protected $_errors;

    /**
     * Class constructor
     */
    public function __construct($input = null)
    {
        global $page;
        $this->_checkoutDir = dirname(__FILE__) . "/checkout/";
        $this->_webRoot = _SYSTEM_BASE_URL._SRV_WEB._SRV_WEB_PLUGINS.get_class($this).'/';

        $this->allowed_subpayments = array(
            'part', 'invoice', 'spec'
        );

        $this->api = new XtKlarna();

        $this->fillData();

        if (!$this->getConfig()) {
            //Not properly configured!
            return;
        }

        $this->init();

        $this->handlePageAction($page->page_name, $page->page_action);
    }

    /**
     * Class destructor
     */
    public function __destruct()
    {
    }

    /**
     * (Re-)Initializes the API object.
     *
     * @return void
     */
    protected function init()
    {
        $this->api->config($this->_eid, $this->_secret, null, null, null, $this->_mode, 'mysql', self::getPcURI());
    }

    /**
     * Used to store pclasses in the database
     */
    public static function getPcURI() {
        $pcURI = array( 'user' => _SYSTEM_DATABASE_USER,
                        'passwd' => _SYSTEM_DATABASE_PWD,
                        'dsn' => _SYSTEM_DATABASE_HOST,
                        'db' => _SYSTEM_DATABASE_DATABASE,
                        'table' => DB_PREFIX . '_klarna_pclasses'
                        );
        return $pcURI;
    }

    /**
     * Retrieves the config values for the module.
     *
     * @return bool, true if config OK.
     */
    protected function getConfig()
    {
        $country = strtoupper($this->api->getCountryCode($this->data['use_country']));
        $this->_eid = constant('XT_KLARNA_EID_'.$country);
        $this->_secret = constant('XT_KLARNA_SECRET_'.$country);
        $this->_mode = XT_KLARNA_MODE;

        if ($this->_eid <= 0 || !is_numeric($this->_eid)) {
            return false;
        } else if (strlen($this->_secret) == 0) {
            return false;
        }

        $this->data['default_option'] = XT_KLARNA_DEFAULT_OPTION;

        $this->data['show_part'] = XT_KLARNA_ENABLE_PART;
        $this->data['show_invoice'] = XT_KLARNA_ENABLE_INVOICE;
        $this->data['show_spec'] = XT_KLARNA_ENABLE_SPEC;

        return true;
    }

    /**
     * Fills the data array for the templates, etc.
     *
     * @return void
     */
    protected function fillData()
    {
        global $language, $currency;

        $this->data['payment_icon'] = $this->getIcon();
        $this->data['payment_name'] = 'Klarna';

        // Circumvent the selected payment sub name.
        define('TEXT_PAYMENT_PART', $this->data['payment_name']);
        define('TEXT_PAYMENT_INVOICE', $this->data['payment_name']);
        define('TEXT_PAYMENT_SPEC', $this->data['payment_name']);

        $this->data['img_path'] = 'images/klarna/';
        $this->data['img_path'] = $this->_webRoot . 'images/klarna/';

        $this->data['country_code'] = _STORE_COUNTRY;
        $this->data['language_code'] = $language->code;
        $this->data['currency_code'] = $currency->code;
        try {
            $this->data['use_currency'] = XtKlarna::getCurrencyForCode($currency->code);
            $this->data['use_country'] = XtKlarna::getCountryForCode(_STORE_COUNTRY);
        } catch(KlarnaException $e) {
            //Currency or country not supported.
            $this->data['show_part'] = false;
            $this->data['show_spec'] = false;
            $this->data['show_invoice'] = false;
        }

        //Prefill data
        $this->data['dob'] = '';
        $this->data['gender'] = '';
        $this->data['company'] = '';
        $this->data['fname'] = '';
        $this->data['lname'] = '';
        $this->data['phone'] = '';
        $this->data['zip'] = '';
        $this->data['city'] = '';
        $this->data['street'] = '';
        $this->data['houseno'] = '';
        $this->data['houseext'] = '';

        if ($_SESSION['customer']->customers_id > 0) {
            $addr = (empty($_SESSION['customer']->customer_shipping_address) ?
                $_SESSION['customer']->customer_default_address : $_SESSION['customer']->customer_shipping_address);
            if ($addr['customers_phone'] == '') {
                $sPhone = $_SESSION['customer']->customer_default_address['customers_phone'];
            } else {
                $sPhone = $addr['customers_phone'];
            }
            $this->data['dob']      = $addr['customers_dob']; //dd.mm.yyyy
            $this->data['gender']   = $addr['customers_gender'];
            $this->data['company']  = $addr['customers_company'];
            $this->data['fname']    = $addr['customers_firstname'];
            $this->data['lname']    = $addr['customers_lastname'];
            $this->data['email']    = $_SESSION['customer']->customer_info['customers_email_address'];
            $this->data['phone']    = $sPhone;
            $this->data['street']   = $addr['customers_street_address'];
            $this->data['zip']      = $addr['customers_postcode'];
            $this->data['city']     = $addr['customers_city'];
            $this->data['country_code'] = $addr['customers_country_code'];
            $this->data['use_country'] = XtKlarna::getCountryForCode($addr['customers_country_code']);

            if (KlarnaCountry::DE == $this->data['use_country']
                    || KlarnaCountry::NL == $this->data['use_country']) {
                $splitAddr = KlarnaAPI::splitAddress($this->data['street']);
                $this->data['street'] = $splitAddr[0];
                $this->data['houseno'] = $splitAddr[1];
                if (KlarnaCountry::NL == $this->data['use_country']) {
                    $this->data['houseext'] = $splitAddr[2];
                }
            }
        }

        try {
            $this->data['use_language'] = XtKlarna::getLanguageForCode($language->code);
        }
        catch(Exception $e) {
            //Unsupported language
            $this->data['use_language'] = $this->api->getLanguageForCountry($this->data['use_country']);
        }

        if (isset($_SESSION['klarna'])) {
            $this->data['dob'] = $_SESSION['klarna']['dob'];
        }
    }

    /**
     * Executed if no session data is available or if the shipping address is changed.
     *
     * @return void
     */
    protected function updateData() {
        global $currency;

        $dob = explode('.', $this->data['dob']);
        if ($this->data['use_country'] !== KlarnaCountry::DE && $this->data['use_country'] !== KlarnaCountry::NL) {
            $this->data['dob'] = '';
        }
        $_SESSION['klarna'] = array(
            'address'   => isset($_SESSION['klarna']['address']) ? $_SESSION['klarna']['address'] : '',
            'dob'       => isset($_SESSION['klarna']['dob']) ? $_SESSION['klarna']['dob'] : $this->data['dob'],
            'company'   => $this->data['company'],
            'fname'     => $this->data['fname'],
            'lname'     => $this->data['lname'],
            'phone'     => $this->data['phone'],
            'gender'    => $this->data['gender'],
            'pclass'    => isset($_SESSION['klarna']['pclass']) ? $_SESSION['klarna']['pclass'] : -1, //Invoice PClass ID
            "day"       => isset($_SESSION['klarna']['day']) ? $_SESSION['klarna']['day'] : $dob[0],
            "month"     => isset($_SESSION['klarna']['month']) ? $_SESSION['klarna']['month'] : $dob[1],
            "year"      => isset($_SESSION['klarna']['year']) ? $_SESSION['klarna']['year'] : $dob[2],
            'zip'       => $this->data['zip'],
            'city'      => $this->data['city'],
            'street'    => $this->data['street'],
            'houseNo'   => $this->data['houseno'],
            'houseExt'  => $this->data['houseext'],
            'email'     => $this->data['email'],
            'reference' => $this->data['fname'].' '.$this->data['lname'],
            'prefix'    => strval($currency->prefix),
            'suffix'    => strval($currency->suffix),
            'ysal'      => isset($_SESSION['klarna']['ysal']) ? $_SESSION['klarna']['ysal'] : null,
        );
    }

    /**
     * Checks whether PClasses are available.
     *
     * @param  array  $pcTypes
     * @return bool
     */
    protected function hasPClasses($pcTypes, $sum)
    {
        if (is_array($pcTypes)) {
            $pclassCount = 0;
            foreach ($pcTypes as $pcType) {
                $pclasses = $this->api->getPClasses($pcType);
                foreach ($pclasses as $pclass) {
                    if ($sum >= $pclass->getMinAmount()) {
                        $pclassCount += 1;
                    }
                }
            }
            if ($pclassCount == 0) {
                return false;
            }
        }

        return true;
    }

    /**
     * Generates the inputs for the standard checkout and validation.
     *
     * @param  string  $option  Invoice, spec or part.
     * @return void
     */
    protected function generateInputs($option)
    {
        if (!in_array($option, $this->allowed_subpayments)) {
            return;
        }

        $type = '';
        if ($option == 'invoice') {
            $type = '_inv';
        } else if ($option == 'part') {
            $type = '';
        } else if ($option == 'spec') {
            $type = '_spec';
        }

        switch($this->data['use_country']) {
            case KlarnaCountry::SE:
                $this->_inputs = array(
                    "socialNumber"          => "klarna{$type}_pnum",
                    "phoneNumber"           => "klarna{$type}_phone",
                    "emailAddress"          => "klarna{$type}_email",
                    "invoiceType"           => "klarna{$type}_invoice_type",
                    "reference"             => "klarna{$type}_reference",
                    "shipmentAddressInput"  => "klarna{$type}_shipment_address"
                );
                break;
            case KlarnaCountry::DE:
                $this->_inputs = array(
                    "firstName"             => "klarna{$type}_de_fname",
                    "lastName"              => "klarna{$type}_de_lname",
                    "gender"                => "klarna{$type}_de_gender",
                    "street"                => "klarna{$type}_de_street",
                    "homenumber"            => "klarna{$type}_de_house",
                    "zipcode"               => "klarna{$type}_de_postno",
                    "city"                  => "klarna{$type}_de_city",
                    "phoneNumber"           => "klarna{$type}_de_phone",
                    "emailAddress"          => "klarna{$type}_de_email",
                    "invoiceType"           => "klarna{$type}_invoice_type",
                    "reference"             => "klarna{$type}_reference",
                    "birth_day"             => "klarna{$type}_birth_day",
                    "birth_month"           => "klarna{$type}_birth_month",
                    "birth_year"            => "klarna{$type}_birth_year",
                    "consent"               => "consent",
                    "shipmentAddressInput"  => "klarna{$type}_shipment_address"
                );
                break;
            case KlarnaCountry::NL:
                $this->_inputs = array(
                    "firstName"             => "klarna{$type}_nl_fname",
                    "lastName"              => "klarna{$type}_nl_lname",
                    "gender"                => "klarna{$type}_nl_gender",
                    "street"                => "klarna{$type}_nl_street",
                    "homenumber"            => "klarna{$type}_nl_house",
                    "house_extension"       => "klarna{$type}_nl_houseext",
                    "zipcode"               => "klarna{$type}_nl_postno",
                    "city"                  => "klarna{$type}_nl_city",
                    "phoneNumber"           => "klarna{$type}_nl_phone",
                    "emailAddress"          => "klarna{$type}_nl_email",
                    "invoiceType"           => "klarna{$type}_invoice_type",
                    "reference"             => "klarna{$type}_reference",
                    "birth_day"             => "klarna{$type}_birth_day",
                    "birth_month"           => "klarna{$type}_birth_month",
                    "birth_year"            => "klarna{$type}_birth_year",
                    "shipmentAddressInput"  => "klarna{$type}_shipment_address"
                );
                break;
            case KlarnaCountry::DK:
                $this->_inputs = array(
                    "companyName"           => "klarna{$type}_dk_company",
                    "socialNumber"          => "klarna{$type}_dk_pnum",
                    "invoice_type"          => "klarna{$type}_dk_invoice_type",
                    "firstName"             => "klarna{$type}_dk_fname",
                    "lastName"              => "klarna{$type}_dk_lname",
                    "gender"                => "klarna{$type}_dk_gender",
                    "street"                => "klarna{$type}_dk_street",
                    "homenumber"            => "klarna{$type}_dk_house",
                    "zipcode"               => "klarna{$type}_dk_postno",
                    "city"                  => "klarna{$type}_dk_city",
                    "phoneNumber"           => "klarna{$type}_dk_phone",
                    "emailAddress"          => "klarna{$type}_dk_email",
                    "invoiceType"           => "klarna{$type}_invoice_type",
                    "reference"             => "klarna{$type}_dk_reference",
                    "shipmentAddressInput"  => "klarna{$type}_shipment_address",
                    "year_salary"           => "ysal{$type}"
                );
                break;
            case KlarnaCountry::FI:
                $this->_inputs = array(
                    "companyName"           => "klarna{$type}_fi_company",
                    "socialNumber"          => "klarna{$type}_fi_pnum",
                    "invoice_type"          => "klarna{$type}_fi_invoice_type",
                    "firstName"             => "klarna{$type}_fi_fname",
                    "lastName"              => "klarna{$type}_fi_lname",
                    "gender"                => "klarna{$type}_fi_gender",
                    "street"                => "klarna{$type}_fi_street",
                    "homenumber"            => "klarna{$type}_fi_house",
                    "zipcode"               => "klarna{$type}_fi_postno",
                    "city"                  => "klarna{$type}_fi_city",
                    "phoneNumber"           => "klarna{$type}_fi_phone",
                    "emailAddress"          => "klarna{$type}_fi_email",
                    "invoiceType"           => "klarna{$type}_invoice_type",
                    "reference"             => "klarna{$type}_fi_reference",
                    "shipmentAddressInput"  => "klarna{$type}_shipment_address"
                );
                break;
            case KlarnaCountry::NO:
                $this->_inputs = array(
                    "companyName"           => "klarna{$type}_no_company",
                    "socialNumber"          => "klarna{$type}_no_pnum",
                    "invoice_type"          => "klarna{$type}_no_invoice_type",
                    "firstName"             => "klarna{$type}_no_fname",
                    "lastName"              => "klarna{$type}_no_lname",
                    "gender"                => "klarna{$type}_no_gender",
                    "street"                => "klarna{$type}_no_street",
                    "homenumber"            => "klarna{$type}_no_house",
                    "zipcode"               => "klarna{$type}_no_postno",
                    "city"                  => "klarna{$type}_no_city",
                    "phoneNumber"           => "klarna{$type}_no_phone",
                    "emailAddress"          => "klarna{$type}_no_email",
                    "invoiceType"           => "klarna{$type}_invoice_type",
                    "reference"             => "klarna{$type}_no_reference",
                    "shipmentAddressInput"  => "klarna{$type}_shipment_address"
                );
                break;
        }
        if ($option == 'part' || $option == 'spec') {
            $this->_inputs['paymentPlan'] = "pclass{$type}";
        }
    }

    /**
     * Creates the HTML for the checkout page.
     *
     * @param  string  $option  Invoice, part or spec.
     * @return string  HTML
     */
    protected function getCheckoutPage($option)
    {
        global $language, $price, $currency;

        if (!$this->data['show_'.$option]) {
            //Payment module is disabled.
            return;
        }
        $totalSum = $_SESSION['cart']->total['plain'];

        //No ILT tests, so do not show Klarna for amounts above 250 euro.
        if ($this->data['use_country'] == KlarnaCountry::NL && $totalSum > 250) {
            $this->data['show_'.$option] = false;
            return;
        }

        $lang = $this->api->getLanguageForCountry();

        //Set pclass types
        $pcTypes = null;
        if ($option == 'part') {
            $pcTypes = array(KlarnaPClass::ACCOUNT, KlarnaPClass::CAMPAIGN, KlarnaPClass::FIXED);
        } else if($option == 'spec') {
            $pcTypes = array(KlarnaPClass::SPECIAL);
        }

        //Check if we have any pclasses for specified types available
        if ( !$this->hasPClasses($pcTypes, $totalSum)) {
            //No pclasses, hide spec/part.
            $this->data['show_'.$option] = false;
            return;
        }

        $pclass = -1;
        //Set payment option title
        if ($option == 'part') {
            $pclass = $this->api->getCheapestPClass($totalSum, KlarnaFlags::CHECKOUT_PAGE);
            $cheapest = 0.0;
            if ($pclass) {
                $cheapest = KlarnaCalc::calc_monthly_cost($totalSum, $pclass, KlarnaFlags::CHECKOUT_PAGE);
                Klarna::printDebug('checkout cheapest pclass', $cheapest);
            }
            $pcl = $this->api->getPClasses(KlarnaPClass::ACCOUNT);
            if (count($pcl) > 0) {
                $pclass = $pcl[0];
            }
            $format = new xt_format();
            $formattedPrice = $format->formatPrice($cheapest);
            $this->data['part_payment_name'] = str_replace('xx', $formattedPrice, $this->getPaymentTitle('part'));
        } else if ($option == 'invoice') {
            $this->data['invoice_payment_name'] = $this->getPaymentTitle('invoice');
        } else if($option == 'spec') {
            //Grab the last PClass type (should only be one)
            $pcl = $this->api->getPClasses(end($pcTypes));
            if (count($pcl) > 0) {
                $pclass = $pcl[0];
                $this->data['spec_payment_name'] = $pcl[0]->getDescription();
            } else {
                $this->data['show_spec'] = false;
            }
        }

        //Prefill data in session
        if (!isset($_SESSION['klarna'])) {
            $this->updateData();
        }

        //Pick the chosen template
        $template = null;
        if ($option == 'spec') {
            $template = array(
                'name' => 'default'
            );
        }
        //Try to grab the checkout HTML.
        try {
            $checkout = new KlarnaAPI(
                $this->data['country_code'], $lang,
                $option, $totalSum, KlarnaFlags::CHECKOUT_PAGE, $this->api, $pcTypes, $this->_checkoutDir
            );
            $checkout->setFormatter(new xt_format());
            $fee = 0;
            if ($option == 'invoice') {
                $fee = $this->getPaymentFee();
            }

            $checkout->addMultipleSetupValues(
                array(
                    'eid' => $this->_eid,
                    'path_css' => $this->_webRoot . 'images/',
                    'path_js' => $this->_webRoot . 'javascript/',
                    'path_img' => $this->_webRoot . 'images/',
                    'fee' => $fee,
                    'ajax_path' => $this->_webRoot . "klarnaAjax.php",
                    'payment_id' => 'selected_payment',
                    'invoice_name' => 'xt_klarna:invoice',
                    'part_name' => 'xt_klarna:part',
                    'spec_name' => 'xt_klarna:spec',
                    'currency_suffix' => strval($currency->suffix),
                    'currency_prefix' => strval($currency->prefix)
                )
            );
            //Form input id/name's
            $this->generateInputs($option);
            $aParams = $this->_inputs;

            $checkout->addSetupValue('agb_link', _SYSTEM_BASE_URL . _SRV_WEB . $_SESSION['selected_language'] . '/content?coID=' . XT_KLARNA_AGB);

            $pcId = -1;

            // Pre-select chosen pclass
            if ($option != 'invoice' && (count($pcl) > 0) ) {
                if ( $_SESSION['klarna']['pclass']  != -1 ) {
                    // selected pclass
                    $selpc = $this->api->getPClass($_SESSION['klarna']['pclass']);

                    if (in_array($selpc->getType(), $pcTypes)) {
                        $pcId = $selpc->getId();
                    }
                }
                if ($pcId === -1) {
                    $pcId = $pclass->getId();
                }
            }

            $aValues = array(
                "firstName"             => utf8_decode($_SESSION['klarna']['fname']),
                "lastName"              => utf8_decode($_SESSION['klarna']['lname']),
                "phoneNumber"           => utf8_decode($_SESSION['klarna']['phone']),
                "paymentPlan"           => ($pcId),
                "gender"                => ($_SESSION['klarna']['gender']),
                "zipcode"               => utf8_decode($_SESSION['klarna']['zip']),
                "city"                  => utf8_decode($_SESSION['klarna']['city']),
                "street"                => utf8_decode($_SESSION['klarna']['street']),
                "homenumber"            => utf8_decode($_SESSION['klarna']['houseNo']),
                "house_extension"       => utf8_decode($_SESSION['klarna']['houseExt']),
                "birth_day"             => ($_SESSION['klarna']['day']),
                "birth_month"           => ($_SESSION['klarna']['month']),
                "birth_year"            => ($_SESSION['klarna']['year']),
                "emailAddress"          => utf8_decode($_SESSION['klarna']['email']),
                "reference"             => utf8_decode($_SESSION['klarna']['reference']),
                "companyName"           => utf8_decode($_SESSION['klarna']['company']),
                "shipmentAddressInput"  => utf8_decode($_SESSION['klarna']['address']),
                "year_salary"           => ($_SESSION['klarna']['ysal'])
            );

            if (isset($_SESSION['klarna_error_'.$option])) {
                $error = addslashes($_SESSION['klarna_error_'.$option]);
                unset($_SESSION['klarna_error_'.$option]);
                $this->data['default_option'] = $option;
                $checkout->addSetupValue('red_baloon_content', $error);
                $checkout->addSetupValue('red_baloon_paymentBox', 'klarna_box_'.$option);
            }

            $html = $checkout->retrieveHTML($aParams, $aValues, null, $template);
        }
        catch(KlarnaApiException $e) {
            //Something went wrong, hide the payment option
            $html =  $e->__toString();
            $this->data['show_'.$option] = false;
        }

        return $html;
    }

    /**
     * Creates the HTML for the product page.
     *
     * @param  float   $totalSum
     * @return string  HTML
     */
    public function getProductPage($totalSum)
    {
        global $p_info, $currency, $price;

        Klarna::printDebug(__METHOD__, null);

        if (!$this->data['show_part']) {
            //Part payment is disabled.
            return;
        }

        $lang = $this->api->getLanguageForCountry();
        //No ILT tests, so do not show Klarna for amounts above 250 euro.
        if ($this->data['use_country'] == KlarnaCountry::NL && $totalSum > 250) {
            $this->data['show_part'] = false;
            return;
        }

        $pcTypes = array(
            KlarnaPClass::ACCOUNT,
            KlarnaPClass::CAMPAIGN
        );

        //Check if we have any pclasses for specified types available
        if ( !$this->hasPClasses($pcTypes, $totalSum)) {
            //No pclasses, hide spec/part.
            return "";
        }

        try {
            $checkout = new KlarnaAPI(
                $this->data['country_code'], $lang,
                'part', $totalSum, KlarnaFlags::PRODUCT_PAGE, $this->api, $pcTypes,
                $this->_checkoutDir
            );

            $checkout->addMultipleSetupValues(
                array(
                  'eid' => $this->_eid,
                  'path_css' => $this->_webRoot . 'images/',
                  'path_js'  => $this->_webRoot . 'javascript/',
                  'path_img' => $this->_webRoot . 'images/',
                  'asterisk'  => ($this->data['use_country'] == KlarnaCountry::DE) ? '*' : '',
                )
            );

            $ppbox = new KlarnaProductPrice($this->api, $this->_eid, $this->_checkoutDir, $this->_webRoot . 'images/', $checkout);
            $ppbox->__setFormatter(new xt_format());

            return $ppbox->show($totalSum, $this->api->getCurrency(), $this->api->getCountry(), $lang);
        }
        catch(Exception $e) {
        }
    }

    /**
     * Performs various actions depending on where in the checkout process
     * the end customer is.
     *
     * @param  string  $page
     * @param  string  $action
     * @return void
     */
    protected function handlePageAction($page, $action)
    {
        global $order;

        if (!$this->api->showPayment()) {
            return;
        }
        if ($page == 'checkout') {
            //Sometimes the 'payment' stage can be 'confirmation', bug in Veyton?
            if ($action == 'payment' || $action == 'confirmation') {
                try {
                    $this->data['selected_payment_sub'] = $_SESSION['selected_payment_sub'];
                    if ($this->data['show_spec']) {
                        $this->data['spec_html'] = $this->getCheckoutPage('spec');
                    }
                    if ($this->data['show_part']) {
                        $this->data['part_html'] = $this->getCheckoutPage('part');
                    }
                    if ($this->data['show_invoice']) {
                        $this->data['invoice_html'] = $this->getCheckoutPage('invoice');
                    }

                    //If there is only one payment plugin, make it default
                    $payment = new payment();
                    $plugins = $payment->_getPossiblePayment();
                    if (count($plugins) == 1) {
                        //If there is no defaulted option or it isn't shown
                        if (!$this->data['show_'.$this->data['default_option']]) {
                            //Pick default according to below priority
                            if ($this->data['show_spec']) {
                                $this->data['default_option'] = 'spec';
                            } else if ($this->data['show_part']) {
                                $this->data['default_option'] = 'part';
                            } else if ($this->data['show_invoice']) {
                                $this->data['default_option'] = 'invoice';
                            }
                        }
                    }
                }
                catch(Exception  $e) {
                }
            }

            if ($action == 'confirmation' && $_SESSION['selected_payment'] == get_class($this)) {
                Klarna::printDebug('confirmation invoice fee', null);
                if ($_SESSION['selected_payment_sub'] == 'invoice') {
                    $_SESSION['cart']->show_sub_content['payment']['products_name'] = $this->getInvoiceFeeTitle();
                }
            }

            if ($action == 'shipping') {
                $this->updateData();
            }
        }
    }

    /**
     * Processes the SESSION address variable which is set by klarnaAjax.
     * If an address is found for the specified pno, it will compare
     * it with the specified address and return true if they match.
     *
     * @param  string $pno
     * @param  string $address  Address from POST variable.
     * @return bool
     */
    protected function processAddress($pno, $address)
    {
        if (strlen($address) == 0) {
            //No address string
            return false;
        }

        $aSessionCalls = unserialize(base64_decode($_SESSION['klarna_address']));
        Klarna::printDebug(__METHOD__, $aSessionCalls);
        $expected = array();
        $addrs = $aSessionCalls[$pno];
        if(count($addrs) == 0) {
            //No addresses for specified pno
            return false;
        }
        foreach ($addrs as $addr) {
            Klarna::printDebug('pno addr', $addr);
            $tmp = array();
            if($addr->isCompany === true) {
                $tmp[] = $addr->getCompanyName();
            } else {
                $tmp[] = $addr->getFirstName();
                $tmp[] = $addr->getLastName();
            }

            $tmp[] = $addr->getStreet();
            $tmp[] = $addr->getZipCode();
            $tmp[] = $addr->getCity();
            $tmp[] = KlarnaCountry::getCode($addr->getCountry());

            $expected[] = Klarna::num_htmlentities(implode('|', $tmp));
        }

        Klarna::printDebug('address', $address);
        Klarna::printDebug('num address', Klarna::num_htmlentities(utf8_decode($address)));
        Klarna::printDebug('expected', $expected);

        //Test with different encodings, in case it fails on one.
        if (in_array($address, $expected)) {
            return true;
        } else if(in_array(Klarna::num_htmlentities($address), $expected)) {
            return true;
        } else if(in_array(Klarna::num_htmlentities(utf8_decode($address)), $expected)) {
            return true;
        } else if(in_array(Klarna::num_htmlentities(utf8_encode($address)), $expected)) {
            return true;
        }
        return false;
    }

    /**
     * Translates a key into a real POST field
     * and returns the value for that field.
     *
     * @param  string  $key POST key
     * @return string  POST value
     */
    protected function _POST($key)
    {
        if (isset($this->_inputs[$key])) {
            $key = $this->_inputs[$key];
            if (array_key_exists($key, $_POST)) {
                return trim($_POST[$key]);
            }
        }
        return null;
    }

    /**
     * Validates the various inputs from the payment page.
     *
     * @return void
     */
    public function validateInput()
    {
        global $xtLink;

        //invoice, part or spec?
        $option = substr($_POST['selected_payment'], strpos($_POST['selected_payment'], ':')+1);

        $this->generateInputs($option);
        $lang = $this->api->getLanguageForCountry();

        $this->_errorMsg = array(
            'socialNumber' => $this->fetchLanguage("klarna_personalOrOrganisatio_number", $lang),
            'orgNumber' => $this->fetchLanguage("klarna_personalOrOrganisatio_number", $lang),
            'homenumber' => $this->fetchLanguage('address_homenumber', $lang),
            'companyName' => $this->fetchLanguage('company_name', $lang),
            'firstName' => $this->fetchLanguage("first_name", $lang),
            'lastName' => $this->fetchLanguage("last_name", $lang),
            'street' => $this->fetchLanguage("address_street", $lang),
            'city' => $this->fetchLanguage("address_city", $lang),
            'pno' => $this->fetchLanguage("klarna_personalOrOrganisatio_number", $lang),
            'zipcode' => $this->fetchLanguage("address_zip", $lang),
            'emailAddress' => $this->fetchLanguage("klarna_email", $lang),
            'phoneNumber' => $this->fetchLanguage("phone_number", $lang),
            'gender' => $this->fetchLanguage("sex", $lang),
            'consent' => $this->fetchLanguage("no_consent", $lang),
            'ysal' => $this->fetchLanguage('year_salary', $lang),
            'get_address' => $this->fetchLanguage("no_get_address", $lang),
            'dob' => $this->fetchLanguage('birthday', $lang)
        );

        $inputs = array_flip($this->_inputs);

        //Validate all POST inputs.
        $this->_errors = array();
        $companyPurchase = null;

        //Check invoice type (company or private)
        $invoiceType = $this->_POST('invoiceType');

        $_SESSION['klarna']['type'] = $invoiceType;
        if ($invoiceType !== null) {
            $companyPurchase = ($invoiceType == 'company') ? true : false;
        }

        $reference = $this->_POST('reference');
        $_SESSION['klarna']['reference'] = $reference;

        $pclassId = $this->_POST('paymentPlan');
        $_SESSION['klarna']['pclass'] = ($pclassId) ? $pclassId : -1;

        $this->validateSSN();
        $this->validatePhone();
        $this->validateEmail();

        if ($this->data['use_country'] != KlarnaCountry::SE) {
            $this->validateCompanyName($companyPurchase);
            $this->validateFirstName($companyPurchase);
            $this->validateLastName($companyPurchase);
            $this->validateStreet();
            $this->validateCity();
            $this->validateZip();
        }

        if ($this->data['use_country'] == KlarnaCountry::DE || $this->data['use_country'] == KlarnaCountry::NL) {
            $this->validateGender();
            $this->validateHouseNumber();
        }

        if ($this->data['use_country'] == KlarnaCountry::DE) {
            $this->validateConsent();
        }
        if ($option == 'part') {
            if ( $this->data['use_country'] == KlarnaCountry::DK ) {
                $this->validateSalary();
            }
        }

        $errors = array_unique($this->_errors);

        if (count($errors) > 0) {
            $_SESSION['klarna_error_'.$option] =
                $this->fetchLanguage('error_title_1', $lang).
                '<br/><ul style="list-style: square inside none ! important;"><li>'.implode('</li><li>', $errors).
                '</li></ul><br/>'.$this->fetchLanguage('error_title_2', $lang);
        }

        unset($this->_errors);

        return (count($errors) == 0);
    }

    /**
     * Validates the SSN/DOB.
     *
     * @return void
     */
    protected function validateSSN()
    {
        if ($this->data['use_country'] == KlarnaCountry::DE || $this->data['use_country'] == KlarnaCountry::NL) {
            $_SESSION['klarna']['day'] = $this->_POST('birth_day');
            $_SESSION['klarna']['month'] = $this->_POST('birth_month');
            $_SESSION['klarna']['year'] = $this->_POST('birth_year');
            $dob = $_SESSION['klarna']['day'].$_SESSION['klarna']['month'].$_SESSION['klarna']['year'];
            $_SESSION['klarna']['dob'] = $dob;
            if ( !checkdate($_SESSION['klarna']['month'], $_SESSION['klarna']['day'], $_SESSION['klarna']['year']) ||
                 !KlarnaEncoding::checkPNO($dob, $this->api->getPNOEncoding()) ) {
                $this->_errors[] = $this->_errorMsg['dob'];
            }
        } else {
            $pno = $this->_POST('socialNumber');
            $_SESSION['klarna']['dob'] = $pno;
            if (!KlarnaEncoding::checkPNO($pno, $this->api->getPNOEncoding())) {
                $this->_errors[] = ($companyPurchase) ? $this->_errorMsg['orgNumber'] : $this->_errorMsg['socialNumber'];
                return; //Skip the address check below if invalid PNO.
            }

            if ($this->data['use_country'] == KlarnaCountry::SE) {
                //Do we have the address array set by KlarnaAjax?
                if (!array_key_exists('klarna_address', $_SESSION)) {
                    $this->_errors[] = $this->_errorMsg['get_address'];
                } else {
                    $shipmentAddress = $this->_POST('shipmentAddressInput');
                    if ($this->processAddress($pno, $shipmentAddress) === true) {
                        $_SESSION['klarna']['address'] = $shipmentAddress;
                    } else {
                        //No address match!
                        $this->_errors[] = $this->_errorMsg['get_address'];
                    }
                }
            }
        }
    }

    /**
     * Validates the company name.
     *
     * @param  bool $companyPurchase
     * @return void
     */
    protected function validateCompanyName($companyPurchase)
    {
        $companyName = $this->_POST('companyName');
        $_SESSION['klarna']['company'] = $companyName;
        if ($companyPurchase && !XtKlarna::validate_string($companyName)) {
            $this->_errors[] = $this->_errorMsg['companyName'];
        }
    }

    /**
     * Validates the first name.
     *
     * @param  bool $companyPurchase
     * @return void
     */
    protected function validateFirstName($companyPurchase)
    {
        $firstName = $this->_POST('firstName');
        $_SESSION['klarna']['fname'] = strval($firstName);
        if (!XtKlarna::validate_string($firstName) && !$companyPurchase) {
            $this->_errors[] = $this->_errorMsg['firstName'];
        }
    }

    /**
     * Validates the last name.
     *
     * @param  bool $companyPurchase
     * @return void
     */
    protected function validateLastName($companyPurchase)
    {
        $lastName = $this->_POST('lastName');
        $_SESSION['klarna']['lname'] = $lastName;
        if (!XtKlarna::validate_string($lastName) && !$companyPurchase) {
            $this->_errors[] = $this->_errorMsg['lastName'];
        }

    }

    /**
     * Validates the street address.
     *
     * @return void
     */
    protected function validateStreet()
    {
        $street = $this->_POST('street');
        $_SESSION['klarna']['street'] = $street;
        if (!XtKlarna::validate_string($street)) {
            $this->_errors[] = $this->_errorMsg['street'];
        }
    }

    /**
     * Validates the city.
     *
     * @return void
     */
    protected function validateCity()
    {
        $city = $this->_POST('city');
        $_SESSION['klarna']['city'] = $city;
        if (!XtKlarna::validate_string($city)) {
            $this->_errors[] = $this->_errorMsg['city'];
        }
    }

    /**
     * Validates the house number.
     *
     * @return void
     */
    protected function validateHouseNumber()
    {
        $houseNo = $this->_POST('homenumber');
        $_SESSION['klarna']['houseNo'] = $houseNo;
        if (!XtKlarna::validate_house_number($houseNo)) {
            $this->_errors[] = $this->_errorMsg['homenumber'];
        }
    }

    /**
     * Validates the consent box.
     *
     * @return void
     */
    protected function validateConsent()
    {
        if ($this->_POST('consent') != 'on') {
            $this->_errors[] = $this->_errorMsg['consent'];
        }
    }

    /**
     * Validates the gender selection.
     *
     * @return void
     */
    protected function validateGender()
    {
        $tmp = $this->_POST('gender');
        $gender = (strlen($tmp) > 0) ? $tmp : $this->_POST('gender');
        $_SESSION['klarna']['gender'] = $gender;
        if ( (strtolower($gender) != 'm' && strtolower($gender) != 'f') &&
        ($gender != KlarnaFlags::MALE && $gender != KlarnaFlags::FEMALE) ) {
            $this->_errors[] = $this->_errorMsg['gender'];
        }
    }

    /**
     * Validates the zip code.
     *
     * @return void
     */
    protected function validateZip()
    {
        $error = false;
        $zip = $this->_POST('zipcode');
        $_SESSION['klarna']['zip'] = $zip;
        if (!XtKlarna::validate_zipcode($zip)) {
            $this->_errors[] = $this->_errorMsg['zipcode'];
        }
    }

    /**
     * Validates the email address.
     *
     * @return void
     */
    protected function validateEmail()
    {
        $email = $this->_POST('emailAddress');
        if (!KlarnaEncoding::checkPNO($email, KlarnaEncoding::EMAIL)) {
            $this->_errors[] = $this->_errorMsg['emailAddress'];
        }
    }

    /**
     * Validates the phone number.
     *
     * @return void
     */
    protected function validatePhone()
    {
        $phone = $this->_POST('phoneNumber');
        $_SESSION['klarna']['phone'] = $phone;
        if (strlen($phone) === 0) {
          $this->_errors[] = $this->_errorMsg['phoneNumber'];
        }
    }

    /**
     * Validates the yearly salary.
     *
     * @return void
     */
    protected function validateSalary() {
        $ysal = $this->_POST('year_salary');
        $_SESSION['klarna']['ysal'] = ($ysal) ? $ysal : 0;
        if (!XtKlarna::validate_string($ysal)) {
            $this->_errors[] = $this->_errorMsg['ysal'];
        }
    }

    /**
     * Associates the last invoice with the systems order id.
     *
     * @return void
     */
    public function associateOrder($order)
    {
        try {
            if (isset($_SESSION['klarna_invno'])) {
                $this->api->updateOrderNo($_SESSION['klarna_invno'], $_SESSION['last_order_id']);
                if (isset($_SESSION['klarna_order_status']) && $order->order_data['orders_status_id'] != $_SESSION['klarna_order_status']) {
                    $order->_updateOrderStatus($_SESSION['klarna_order_status'], '', 'false', 'false');
                    unset($_SESSION['klarna_order_status']);
                }
            }
        }
        catch(Exception $e) {
            exit;
        }
    }

    /**
     * Assembles all the necessary information and sends it to Klarna
     * to complete the purhcase.
     *
     * Redirects the user in case of error and displays error message.
     *
     * @param  array  &$order_data
     * @return void
     */
    public function performPurchase(&$order_data)
    {

        global $xtLink;
        $option = $_SESSION['selected_payment_sub'];

        if ($this->data['use_country'] == KlarnaCountry::NL || $this->data['use_country'] == KlarnaCountry::DE) {
            $order_data['billing'] = $order_data['delivery'];
        }

        $cart = $_SESSION['cart'];
        $customer = $_SESSION['customer'];

        try {
            if (!$this->api->showPayment()) {
                throw Exception('Currency/country combination');
            }
            $this->compileOrder($cart);

            $billing = $this->compileAddress($order_data['billing']);
            $shipping = $this->compileAddress($order_data['delivery']);

            $shipping->setEmail(utf8_decode($this->data['email']));
            $billing->setEmail(utf8_decode($this->data['email']));
            $shipping->setTelno(utf8_decode($_SESSION['klarna']['phone']));
            $billing->setTelno(utf8_decode($_SESSION['klarna']['phone']));

            //Overwrite the house number and extension, if possible, for shipping.
            if ($this->data['use_country'] == KlarnaCountry::DE || $this->data['use_country'] == KlarnaCountry::NL) {
                $shipping->setHouseNumber($_SESSION['klarna']['houseNo']);
            }
            if ($this->data['use_country'] == KlarnaCountry::NL) {
                $shipping->setHouseExt($_SESSION['klarna']['houseExt']);
            }

            $this->api->setAddress(KlarnaFlags::IS_BILLING, $billing);
            $this->api->setAddress(KlarnaFlags::IS_SHIPPING, $shipping);

            if ($_SESSION['klarna']['type'] == 'company') {
                $this->api->setReference(utf8_decode($_SESSION['klarna']['reference']), "");
            }

            $gender = null;
            if ($this->data['use_country'] == KlarnaCountry::DE || $this->data['use_country'] == KlarnaCountry::NL) {
                $gender = $_SESSION['klarna']['gender'];
            }

            if ($option == 'part' && $this->data['use_country'] == KlarnaCountry::DK) {
                $ysal = intval($_SESSION['klarna']['ysal'] * 100);
                $this->api->setIncomeInfo('yearly_salary', $ysal);
            }

            $pno = $_SESSION['klarna']['dob'];
            $pclass = ($option == 'invoice') ? -1 : $_SESSION['klarna']['pclass'];

            $result = $this->api->addTransaction(utf8_decode($pno), $gender, KlarnaFlags::RETURN_OCR, $pclass);

            //Used for associateOrder()
            $_SESSION['klarna_invno'] = $result[1];

            if ($result[2] == KlarnaFlags::PENDING) {
                if (is_numeric(XT_KLARNA_PENDING_STATUS)) {
                    $_SESSION['klarna_order_status'] = intval(XT_KLARNA_PENDING_STATUS);
                }
                $comment = $this->fetchLanguage('comment_purchase_pending');
            } else {
                if (is_numeric(XT_KLARNA_ACCEPTED_STATUS)) {
                    $_SESSION['klarna_order_status'] = intval(XT_KLARNA_ACCEPTED_STATUS);
                }
                $comment = $this->fetchLanguage('comment_purchase_accepted');
            }

            $order_data['comments'] .= ((strlen($order_data['comments']) == 0) ? "" : "<br/><br/>");
            $order_data['comments'] .= "$comment<br/>".$this->fetchLanguage('comment_purchase_ref').": #$result[1]";
            if ( $_SESSION['klarna']['type'] == 'company' ) {
                $ref = $_SESSION['klarna']['reference'];
                $order_data['comments'] .= "<br/>Reference: ${ref}";
            }
            $desc = '';
            if ($pclass > 0) {
                $pObj = $this->api->getPClass($pclass);
                if ($pObj != false) {
                    $desc = $pObj->getDescription();
                }
            }
            else {
                $desc =  $this->fetchLanguage('invoice_string');
            }
            $order_data['comments'] .= "\n<br/>" . $this->fetchLanguage('comment_pay_option') . ': ' . $desc;
        }
        catch(Exception $e) {
            $_SESSION['klarna_error_'.$option] .= htmlentities($e->getMessage()) . ' (#' .$e->getCode() .')';
            $xtLink->_redirect($xtLink->_link(array('page'=>'checkout', 'paction'=>'payment','conn'=>'SSL')));
        }
    }

    /**
     * Assembles the address.
     *
     * @param  array  $address  The systems representation of the address
     * @return KlarnaAddr
     */
    protected function compileAddress($address)
    {
        $addr = new KlarnaAddr();

        if (strlen($address['customers_company']) > 0) {
            $addr->setCompanyName(utf8_decode($address['customers_company']));
        }

        if (strlen($address['customers_firstname']) > 0) {
            $addr->setFirstName(utf8_decode($address['customers_firstname']));
        }
        if (strlen($address['customers_lastname']) > 0) {
            $addr->setLastName(utf8_decode($address['customers_lastname']));
        }

        $addr->setCountry(utf8_decode($address['customers_country_code']));
        if ($addr->getCountry() == KlarnaCountry::NL || $addr->getCountry() == KlarnaCountry::DE) {
            $split = KlarnaAPI::splitAddress($address['customers_street_address']);
            $addr->setStreet(utf8_decode($split[0]));
            $addr->setHouseNumber(utf8_decode($split[1]));
            if ($addr->getCountry() == KlarnaCountry::NL && isset($split[2])) {
                $addr->setHouseExt(utf8_decode($split[2]));
            }
        } else {
            $addr->setStreet(utf8_decode($address['customers_street_address']));
        }

        $addr->setZipCode(utf8_decode($address['customers_postcode']));
        $addr->setCity(utf8_decode($address['customers_city']));

        $addr->setTelno(utf8_decode($address['customers_phone']));

        return $addr;
    }

    /**
     * Compiles all the articles and fees into the order data.
     *
     * @param  array  $cart  The system representation of the cart.
     * @return void
     */
    protected function compileOrder($cart)
    {
        //Process the sub products (invoice fee, shipping, etc)
        foreach ($cart->show_sub_content as $key => $prod) {
            //Product type?
            $flags = KlarnaFlags::INC_VAT;
            switch($key) {
                case 'shipping':
                    $flags += KlarnaFlags::IS_SHIPMENT;
                    break;
                case 'payment': //invoice fee
                    $flags += KlarnaFlags::IS_HANDLING;
                    $prod['products_name'] = $this->getInvoiceFeeTitle();
                    break;
                default:
            }

            $qty = $prod['products_quantity'];
            $artNo = $prod['products_key'];
            $title = $prod['products_name'];
            $price = floatval($prod['products_price']['plain']);
            $vat = floatval($prod['products_tax_value']);
            $discount = floatval($prod['products_discount']);

            //Add the product to the API.
            $this->api->addArticle(
                utf8_decode($qty),
                utf8_decode($artNo),
                utf8_decode($title),
                utf8_decode($price),
                utf8_decode($vat),
                utf8_decode($discount),
                utf8_decode($flags)
            );
        }

        //Process the products (normal products)
        foreach ($cart->show_content as $key => $prod) {
            $flags = KlarnaFlags::INC_VAT;

            $qty = $prod['products_quantity'];
            $artNo = $prod['products_key'];
            $title = $prod['products_name'];
            $price = floatval($prod['products_price']['plain']);
            $vat = floatval($prod['products_tax_rate']);
            $discount = floatval($prod['products_discount']);

            //Add the product to the API.
            $this->api->addArticle(
                utf8_decode($qty),
                utf8_decode($artNo),
                utf8_decode($title),
                utf8_decode($price),
                utf8_decode($vat),
                utf8_decode($discount),
                utf8_decode($flags)
            );
        }
    }

    /**
     * Returns the invoice fee title from the language XML.
     *
     * @return string
     */
    public function getInvoiceFeeTitle()
    {
        return $this->fetchLanguage('INVOICE_FEE_TITLE');
    }

    /**
     * Fetches/updates the PClasses.
     *
     * @param  array  Store info
     * @return void
     */
    public function updatePClasses($arr)
    {
        Klarna::$debug = false;
        Klarna::$xmlrpcDebug = false;

        if (!$this->data['show_part'] && !$this->data['show_spec']) {
            //No PClass module enabled
            return;
        }

        try {
            $this->api->clearPClasses();
        }
        catch(Exception $e) {
            //Fail silently
        }

        //Update for all country/eid
        foreach ($arr as $country => $info) {
            try {
                if (!is_int($info['eid']) || $info['eid'] <= 0 || strlen($info['secret']) == 0) {
                    continue;
                }

                $this->_eid = $info['eid'];
                $this->_secret = $info['secret'];
                $this->_mode = $info['mode'];

                //Recreate the API
                $this->init();
                $this->api->setCountry($info['country']);
                $country = $this->api->getCountry();

                $this->api->fetchPClasses($country);
            }
            catch(Exception $e) {
            }
        }
    }

    /**
     * Returns the payment title for the specified option.
     *
     * @return string
     */
    public function getPaymentTitle($option)
    {
        if ($option == 'invoice') {
            return $this->fetchLanguage('INVOICE_TITLE_NO_PRICE', $this->api->getLanguageForCountry($this->data['use_country']));
        } else if ($option == 'part') {
            return $this->fetchLanguage('PARTPAY_TITLE', $this->api->getLanguageForCountry($this->data['use_country']));
        } else {
            return '';
        }
    }

    /**
     * Returns the payment fee/invoice fee.
     *
     * @return float
     */
    public function getPaymentFee()
    {
        global $template;

        return $template->content_smarty->_tpl_vars['PRICE']['plain'];
    }

    /**
     * Grab the specified string from the language XML in the specified language.
     *
     *
     * @param string  $string
     * @param string  $lang
     */
    protected function fetchLanguage($string, $lang = null)
    {
        if ($lang === null) {
            $lang = $this->data['use_language'];
        }
        if (is_numeric($lang)) {
            $lang = $this->api->getLanguageCode($lang);
        }
        return KlarnaAPI::fetchFromLanguagePack($string, $lang, $this->_checkoutDir);
    }

    /**
     * Returns the link for the terms and conditions.
     *
     * @param  string  $type  Invoice, part or spec.
     * @return string
     */
    public function getTermsLink($type)
    {
        $charge = $this->getPaymentFee();

        $subdomain = 'online';
        if ($this->_mode === Klarna::BETA) {
            $subdomain = 'beta-test';
        }

        if ($type == 'invoice') {
            switch($this->data['use_country']) {
                case KlarnaCountry::DK:
                    return 'https://'.$subdomain.'.klarna.com/villkor_de.yaws?eid='.$this->_eid.'&charge='.$charge;
                case KlarnaCountry::DE:
                    return 'https://'.$subdomain.'.klarna.com/villkor_dk.yaws?eid='.$this->_eid.'&charge='.$charge;
                case KlarnaCountry::FI:
                    return 'https://'.$subdomain.'.klarna.com/villkor_fi.yaws?eid='.$this->_eid.'&charge='.$charge;
                case KlarnaCountry::NL:
                    return 'https://'.$subdomain.'.klarna.com/villkor_nl.yaws?eid='.$this->_eid.'&charge='.$charge;
                case KlarnaCountry::NO:
                    return 'https://'.$subdomain.'.klarna.com/villkor_no.yaws?eid='.$this->_eid.'&charge='.$charge;
                case KlarnaCountry::SE:
                    return 'https://'.$subdomain.'.klarna.com/villkor.yaws?eid='.$this->_eid.'&charge='.$charge;
            }
        } else if ($type == 'part') {
            switch($this->data['use_country']) {
                case KlarnaCountry::DK:
                    return 'https://'.$subdomain.'.klarna.com/account_dk.yaws?eid='.$this->_eid;
                case KlarnaCountry::DE:
                    return 'https://'.$subdomain.'.klarna.com/account_de.yaws?eid='.$this->_eid;
                case KlarnaCountry::FI:
                    return 'https://'.$subdomain.'.klarna.com/account_fi.yaws?eid='.$this->_eid;
                case KlarnaCountry::NL:
                    return 'https://'.$subdomain.'.klarna.com/account_nl.yaws?eid='.$this->_eid;
                case KlarnaCountry::NO:
                    return 'https://'.$subdomain.'.klarna.com/account_no.yaws?eid='.$this->_eid;
                case KlarnaCountry::SE:
                    return 'https://'.$subdomain.'.klarna.com/account_se.yaws?eid='.$this->_eid;
            }
        } else if ($type == 'spec') {
            $country = $this->api->getCountryCode($this->data['use_country']);
            $campaign = XT_KLARNA_CAMPAIGN;
            return 'https://static.klarna.com/external/html/'.$campaign.'_'.$country.'.html';
        }
    }


    /**
     * Returns the icon for the payment module.
     *
     * @return string
     */
    public function getIcon()
    {
        return 'icon.png';
    }

    public static function getPaymentInfo() {
        try {
            if ($_SESSION['selected_payment'] == 'xt_klarna') {
                $imgPath = _SYSTEM_BASE_URL._SRV_WEB._SRV_WEB_PLUGINS.'xt_klarna/images/logo/';
                $country = strtolower($_SESSION['customer']->customer_shipping_address['customers_country_code']);

                $subMethod = strtolower($_SESSION['selected_payment_sub']);

                $html = '<div style="float: right; position: relative;">';

                if ($subMethod == 'invoice') {
                    $html .= '<img src="' . $imgPath . $country . '/klarna_invoice.png" />';
                } else {
                    $_eid = constant('XT_KLARNA_EID_'.strtoupper($country));
                    $_secret = constant('XT_KLARNA_SECRET_'.strtoupper($country));
                    $pcURI = self::getPcURI();
                    $klarna = new XtKlarna();
                    $klarna->config($_eid, $_secret, $country, null, null, XT_KLARNA_MODE, "mysql", $pcURI);
                    $pclass = $klarna->getPClass($_SESSION['klarna']['pclass']);
                    if ($subMethod == 'part') {
                        $html .= '<img src="' . $imgPath . $country . '/klarna_account.png" />';
                    } else {
                        $html .= '<img src="' . $imgPath . '/klarna_logo.png" />';
                        $html .= '<br>' . $pclass->getDescription() . '<br>';
                    }
                }
                $html .= '</div><div style="clear: both;"></div>';
                echo $html;
            }
        } catch (Exception $e) {
            Klarna::printDebug('extra-info', $e->getMessage());
        }
    }

    public static function showThankyou() {
        global $checkout_data, $language;
        $orderId = $_SESSION['success_order_id'];
        $sobj = $checkout_data['success_order'];
        if ($sobj->order_data['orders_id'] == $orderId && isset($_SESSION['klarna_invno'])) {
            if (strstr($sobj->order_data['payment_code'],'xt_klarna')) {
                require_once(dirname(__FILE__).'/checkout/classes/class.KlarnaLanguagePack.php');
                $kLang = new KlarnaLanguagePack(dirname(__FILE__).'/checkout/data/language.xml');
                $invtext = $kLang->fetch('INVOICE_CREATED_SUCCESSFULLY_RETURNOCR', $language->code);
                $invtext = str_replace('(xx)', $_SESSION['klarna_invno'], $invtext);
                ?>
                    <div style="border: 1px solid #389FD3; background-color: #BADDF0; padding: 10px; font-family: Arial, Verdana; font-size: 16px; margin: 10px; text-align: center;">
                        <img src='<?php echo _SYSTEM_BASE_URL . _SRV_WEB . _SRV_WEB_PLUGINS. "xt_klarna/images/logo/logo_small.png";?>'  style="float: left;" />
                        <div style="clear: both;"></div>
                        <span>
                            <?php echo $invtext; ?>
                        </span>
                    </div>
                <?php
            }
        }
        unset($_SESSION['klarna_invno']);
    }
}
