<?php
defined('_VALID_CALL') or die('Direct Access is not allowed.');

class chart_reviewbooster_campaign
{

	function _get()
	{
		if (isset($_GET['campaign_id']) && is_numeric($_GET['campaign_id']))
		{
			$campaignId = $_GET['campaign_id'];
		}
		if (!isset($_GET['type']))
		{
			die('wrong chart type');
		}
		
		$this->drawChart($_GET['type'], $campaignId);
	}

	private function drawChart($type, $campaignId = false)
	{
		global $db;
		$graph = new graph();
		$graph->bg_colour = '#FFFFFF';
		
		$year = date('Y');
		$month = date('n');
		$week = date('W');
		
		$statisticsData = $this->getStatisticData($type, $campaignId);
		
		$emails_opened = array();
		$review_pages_accessed = array();
		$reviews_submitted = array();
		$xAxisLabels = array();
		$types = array('emails_opened', 'review_pages_accessed', 'reviews_submitted');
		$yAxisMaxValue = 1;
		switch ($type)
		{
			case 'week':
				$graph->title('Weekly Chart ' . date('d', mktime(0, 0, 0, date('m'), date('d') - date('w') + 1, date('Y'))) . '/' . $month . '/' . $year . '-' . date('d', mktime(0, 0, 0, date('m'), date('d') + (7 - date('w')), date('Y'))) . '/' . $month . '/' . $year, _CHART_TITLE_STYLE);
				$graph->set_x_legend(TEXT_DAY, 12, '#736AFF');
				$graph->set_tool_tip('#key#<br>#val# (' . TEXT_DAY . ':#x_label#)');
				
				$j = 0;
				for ($i = 1 - date('w'); $i <= (7 - date('w')); $i++)
				{
					$dayOfWeek = date('w', mktime(0, 0, 0, date('m'), date('d') + $i, date('Y')));
					$emails_opened[$dayOfWeek] = 0;//rand(0, 10);
					$review_pages_accessed[$dayOfWeek] = 0;//rand(0, 10);
					$reviews_submitted[$dayOfWeek] = 0;//rand(0, 10);
					$xAxisLabels[$j] = date('d', mktime(0, 0, 0, date('m'), date('d') + $i, date('Y')));
					$j++;
				}
				foreach ($statisticsData as $row)
				{
					foreach ($types as $type)
					{
						if ($row[$type] > $yAxisMaxValue)
						{
							$yAxisMaxValue = $row[$type];
						}
						${$type}[($row['dayofweek'] - 1)] = $row[$type];
					}					
				}
				break;
			case 'month':
				$graph->title('Monthly Chart ' . $month . '/' . $year, _CHART_TITLE_STYLE);
				$graph->set_x_legend(TEXT_DAY, 12, '#736AFF');
				$graph->set_tool_tip('#key#<br>#val# (' . TEXT_DAY . ':#x_label#)');
				
				$days = (date('t', mktime(0, 0, 0, date('m'), 1, date('Y'))) + 1);
				for ($i = 1; $i < $days; $i++)
				{
					$emails_opened[] = 0;//rand(0, 10);
					$review_pages_accessed[] = 0;//rand(0, 10);
					$reviews_submitted[] = 0;//rand(0, 10);
					$xAxisLabels[$i] = $i;
				}
				foreach ($statisticsData as $row)
				{
					foreach ($types as $type)
					{
						if ($row[$type] > $yAxisMaxValue)
						{
							$yAxisMaxValue = $row[$type];
						}
						${$type}[$row['dayofmonth'] - 1] = $row[$type];
					}					
				}
				break;
			case 'year':
				$graph->title('Yearly Chart ' . $year, _CHART_TITLE_STYLE);
				$graph->set_x_legend(TEXT_MONTH, 12, '#736AFF');
				$graph->set_tool_tip('#key#<br>#val# (' . TEXT_MONTH . ':#x_label#)');
				
				$months = 12;
				for ($i = 1; $i <= $months; $i++)
				{
					$emails_opened[] = 0;//rand(0, 10);
					$review_pages_accessed[] = 0;//rand(0, 10);
					$reviews_submitted[] = 0;//rand(0, 10);
					$xAxisLabels[$i] = date('F', mktime(0, 0, 0, $i, 1, date('Y')));
				}
				foreach ($statisticsData as $row)
				{
					foreach ($types as $type)
					{
						if ($row[$type] > $yAxisMaxValue)
						{
							$yAxisMaxValue = $row[$type];
						}
						${$type}[$row['month'] - 1] = $row[$type];
					}					
				}
				break;
		}
		
		$graph->set_data($emails_opened);
		$graph->line(1, _CHART_COLOR_2, 'Emails opened', 10);
		
		$graph->set_data($review_pages_accessed);
		$graph->line(1, _CHART_COLOR_11, 'Review pages accessed', 10);
		
		$graph->set_data($reviews_submitted);
		$graph->line(1, _CHART_COLOR_18, 'Reviews submitted', 10);
		
		$graph->set_y_legend(TEXT_AMOUNT, 12, '#736AFF');
		$graph->set_y_max($yAxisMaxValue);
		$graph->y_label_steps(5);
		
		$graph->x_axis_colour('#909090', '#FFFFFF');
		$graph->set_x_labels($xAxisLabels);
		$graph->set_x_label_style(10, '#000000', 0, 1, '#ffffff');
		
		echo $graph->render();
	}

	private function getStatisticData($type, $campaignId = false)
	{
		global $db;
		switch ($type)
		{
			case 'week':
				$periodCondition = ' and (YEAR(statistics.statistics_date) = ' . date('Y') . ' and WEEKOFYEAR(statistics.statistics_date) = ' . date('W') . ')';
				break;
			case 'month':
				$periodCondition = ' and (YEAR(statistics.statistics_date) = ' . date('Y') . ' and MONTH(statistics.statistics_date) = ' . date('n') . ')';
				break;
			case 'year':
				$periodCondition = ' and (YEAR(statistics.statistics_date) = ' . date('Y') . ')';
				break;
		}
		$campaignCondition = '';
		if ($campaignId && is_numeric($campaignId))
		{
			$campaignCondition = ' and (statistics.reviewbooster_campaign_id = ' . $campaignId . ')';
		}
		$data = array();
		$query = 'select sum(statistics.reviewbooster_emails_opened) as reviewbooster_emails_opened,
						 sum(statistics.reviewbooster_pages_accessed) as reviewbooster_pages_accessed,
						 sum(statistics.reviewbooster_reviews_submitted) as reviewbooster_reviews_submitted,
						 dayofweek(statistics.statistics_date) as dayofweek,
						 dayofmonth(statistics.statistics_date) as dayofmonth,
						 month(statistics.statistics_date) as month from ' . TABLE_REVIEWBOOSTER_CAMPAIGNS_STATISTICS . ' statistics, ' . TABLE_REVIEWBOOSTER_CAMPAIGNS . ' campaigns where statistics.reviewbooster_campaign_id=campaigns.reviewbooster_campaign_id' . $periodCondition . $campaignCondition . ' group by statistics.statistics_date';
		$result = $db->Execute($query);
		if ($result->RecordCount() > 0)
		{
			while (!$result->EOF)
			{
				$data[] = array('emails_opened' => $result->fields['reviewbooster_emails_opened'], 'review_pages_accessed' => $result->fields['reviewbooster_pages_accessed'], 'reviews_submitted' => $result->fields['reviewbooster_reviews_submitted'], 'dayofweek' => $result->fields['dayofweek'], 'dayofmonth' => $result->fields['dayofmonth'], 'month' => $result->fields['month']);
				$result->MoveNext();
			}
		}
		$result->Close();
		return $data;
	}
}